/*
  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.

  Copyright (c) 2008-2009 PRO ACTIVE SAS - www.proactive-group.com
   
  Ref_Vicc.c
  -----------

  This is the reference applications that shows how to work with
  ISO/IEC 15693 and NXP ICODE1 cards.
  
  LTX 22/09/2008 : initial release

*/

#include "springprox.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int main(int argc, char **argv)
{
  SWORD rc;
  int i = 0;
  BOOL fast = FALSE;
  TCHAR s_buffer[64];
  DWORD features;
  unsigned char snr[8];
  unsigned char data[256];
  unsigned short len      = 256;
  unsigned short block_sz = 0;

  printf("SpringCard SDK for K531/SpringProx-CF/CSB Legacy\n");
  printf("\n");
  printf("ISO 15693 + NXP ICODE1 reference demo\n");
  printf("-------------------------------------\n");
  printf("www.springcard.com\n\n");

  rc = SPROX_GetLibrary(s_buffer, sizeof(s_buffer));
  if (rc != MI_OK)
  {
    printf("Failed to get API version\n");
    goto done;
  }
  printf("API version : %s\n", s_buffer);
  printf("\n");

  /* Open reader */
  /* ----------- */

  rc = SPROX_ReaderOpen(NULL);
  if (rc != MI_OK)
  {
    printf("Reader not found\n");
    goto done;
  }

  printf("Reader found\n");
  rc = SPROX_ReaderGetDevice(s_buffer, sizeof(s_buffer));
  if (rc == MI_OK)
    printf("Reader found on %s\n", s_buffer);

  rc = SPROX_ReaderGetFirmware(s_buffer, sizeof(s_buffer));
  if (rc == MI_OK)
    printf("Reader firwmare is %s\n", s_buffer);

  rc = SPROX_ReaderGetFeatures(&features);
  if ((rc == MI_OK) && !(features & 0x00000002))
  {
    printf("Warning: this reader is not 15693-enabled\n");
    printf("         all subsequent tests will fail !\n");
  }


  /* -------------- */
  /* TEST ISO 15693 */
  /* -------------- */

  /* Configure reader in ISO 15693 standard mode */
  /* ------------------------------------ */
  rc = SPROX_SetConfig(CFG_MODE_ISO_15693_STD);
  if (rc != MI_OK)
  {
    printf("Failed to configure the reader\n");
    goto close;
  }

  /* RF field ON */
  SPROX_ControlRF(TRUE);

  /* Select any ISO 15693 tag in the RF field */
  /* ------------------------------------------ */
  printf("Waiting for a ISO 15693 tag...\n");

  for (;;)
  {
	rc = SPROX_Iso15693_SelectAny(fast,snr);
	if (rc == MI_OK)
	{
      printf("Found ISO 15693 tag in RF field\n");
      break;
	}

	if (i == 5)
	{
	  fast = TRUE;
	} else if (i == 10)
	{
	  fast = FALSE;
	  i=0;
	}
	i++;
  }

  printf("SNR=");
  for (i = 0; i < 8; i++)
    printf("%02X", snr[i]);
  printf("\n");

  /* Get system information of ISO 15693 tag */
  /* ---------------------------------------- */
  printf("System Information...\n");
  len = 256;
  rc = SPROX_Iso15693_GetSystemInformation(NULL, data, &len);
  if (rc == MI_OK) 
  {
	BYTE  ind = 0;

    if (len > 9)
    {
      printf("UID=");
	  for (i = 1; i < 9; i++)
		printf("%02X", data[i]);
        
      ind = 9;
      if (data[0] & 0x01)
      {
        printf(" DSFID=");
		printf("%02X", data[ind]);
        ind++;
      } 
                  
      if (data[0] & 0x02)
      {
        printf(" AFI=");
		printf("%02X", data[ind]);
        ind++;
      } 
                  
      if (data[0] & 0x04)
      {
        printf("\nNumber of Blocks="); 
        printf("%d",data[ind]+1);
        ind++;
        printf(" Block Size=");
		block_sz = (data[ind]+1) & 0x1F;
        printf("%d",block_sz);
        ind++;
      }
                  
      if (data[0] & 0x08)
      {
        printf("\nIC-Ref=");
		printf("%02X", data[ind]);
      }
	  printf("\n");
    }
  } else
  {
    printf("Not supported or error (%d)\n", rc);
  }

  /* Read block 5 on the tag */
  /* ----------------------- */
  len = 256;
  rc = SPROX_Iso15693_ReadSingleBlock(NULL, 5, data, &len);
  if (rc != MI_OK)
  {
    /* Erreur */
    printf("Failed to read block 5 on the tag (%d)\n", rc);
    goto tag_halt;
  }

  printf("Block 5=");
  for (i = 0; i < len; i++)
    printf("%02X", data[i]);
  printf("\n");

  /* Write block 5 on the tag */
  /* ----------------------- */
  if (block_sz != 0)
	len = block_sz;

  for (i = 0; i < len; i++)
    data[i]++;

  rc = SPROX_Iso15693_WriteBlock(NULL, 5, data, len);
  if (rc != MI_OK)
  {
    /* Erreur */
    printf("Failed to write block 5 on the tag (%d)\n", rc);
    goto tag_halt;
  }
  printf("Write block 5 OK\n");

tag_halt:
  /* ISO 15693 tag stop */
  /* -------------------- */
  rc = SPROX_Iso15693_Halt();
  if (rc != MI_OK)
  {
    printf("HALT failed\n");
    goto close;
  }

  /* RF field OFF */
  SPROX_ControlRF(FALSE);

  /* ----------- */
  /* TEST ICODE1 */
  /* ----------- */

  /* Configure reader in ICODE1 standard mode */
  /* ------------------------------------ */
  rc = SPROX_SetConfig(CFG_MODE_ICODE1_STD);
  if (rc != MI_OK)
  {
    printf("Failed to configure the reader\n");
    goto close;
  }

  /* RF field ON */
  SPROX_ControlRF(TRUE);

  /* Select any ICODE1 tag in the RF field */
  /* ------------------------------------------ */

  printf("Waiting for a ICODE1 tag...\n");

  for (;;)
  {
	rc = SPROX_I1_SelectAny(FALSE,snr);
    if (rc == MI_OK)
	{
      printf("Found ICODE1 tag in RF field\n");
      break;
	}
    /*if (rc != MI_NOTAGERR)
	{
      printf("Failed to select tag\n");
      goto close;
	}*/
  }

  printf("SNR=");
  for (i = 0; i < 8; i++)
    printf("%02X", snr[i]);
  printf("\n");

close:
  /* Close the reader */
  SPROX_ReaderClose();

done:
  /* Display last error */
  if (rc == MI_OK)
  {
    printf("Done\n");
  } else
  {
    printf("%s (%d)\n", SPROX_GetErrorMessage(rc), rc);
  }
  return 0;
}
